//
//  EnxSetting.swift
//  Enx_UIKit_iOS
//
//  Created by jaykumar on 20/03/22.
//

import UIKit
import EnxRTCiOS

@objc
public enum BottomOptions: Int {
    case audio = 1, video,cameraSwitch,audioSwitch,disconnect,groupChat
}
@objc
public enum TopOptions : Int{
    case  menu = 1, requestFloor, requestList
}
@objc
public enum MenuOptions: Int {
    case muteRoom = 1,recording,switchAT,annotation
}
@objc
public enum ParticipantListOptions: Int {
    case audio,video,chat,disconnect
}
// This is for internal used to take a decission where to add aditional user demand button
private enum aditional: Int {
    case top = 1,bottom,both
}
public enum customeNotification : String {
    case TopViewColorUpdate
    case BottomViewColorUpdate
    case TopViewConfigurList
    case BottomViewConfigurList
    case NoMoreOptionAllowed
    case FewOptionPending
    case UpdateOptionView
  
  var description : String {
    switch self {
    case .TopViewColorUpdate: return "TopViewColorUpdate"
    case .BottomViewColorUpdate: return "BottomViewColorUpdate"
    case .TopViewConfigurList: return "TopViewConfigurList"
    case .BottomViewConfigurList: return "BottomViewConfigurList"
    case .NoMoreOptionAllowed: return "NoMoreOptionAllowed"
    case .FewOptionPending: return "FewOptionPending"
    case .UpdateOptionView: return "UpdateOptionView"
    }
  }
}

open class EnxSetting: NSObject {
    
    @objc static public let shared = EnxSetting()
    var chatDatas : [String : [AnyObject]] = [:]
    private(set) var isConnected : Bool = false
    private(set) var topViewColor :UIColor = .darkGray.withAlphaComponent(0.8)
    private(set) var bottomViewColor :UIColor = .white.withAlphaComponent(1.0)
    private(set) var topOptionList : [UIButton] = []
    private(set) var bottomOptionList : [UIButton] = []
    private(set) var roomMode : String = "group"
    private(set) var isModerator : Bool = false
    private(set) var shareOption : [String : String] = [:]
    private(set) var clientID : String!
    private(set) var moreOptionList : [EnxMenuOptionObject] = []
    private(set) var optList : [MenuOptions] = []
    private(set) var customOpt : [Int : UIButton] = [:]
    private(set) var isTopOptCreated : Bool = false
    private(set) var isButtomOptCreated : Bool = false
    private(set) var participentListOpt : [ParticipantListOptions] = []
    
    @objc
    public override init(){
    }
    //save Group Chat Message
    func saveGroupChatData(_ enxMessagfeData : EnxChatModel){
        guard var groupChat = chatDatas["groupchat"] else {
            chatDatas["groupchat"] = [enxMessagfeData]
            return
        }
        groupChat.append(enxMessagfeData)
        chatDatas["groupchat"] = groupChat
    }
    //save Private Chat Message
    func savePrivateChat(_ enxData : EnxChatModel){
        guard var priChat = chatDatas[enxData.senderID] else {
            chatDatas[enxData.senderID] = [enxData]
            return
        }
        priChat.append(enxData)
        chatDatas[enxData.senderID] = priChat
    }
    //Get a particular Chat Data
    func getChat(chatType : String) -> [AnyObject]{
        guard let chatData = chatDatas[chatType] else{
            return []
        }
        return chatData
    }
    //Get all Chat Data
    
    @objc
    func allChatData() -> [String : [Any]]{
        return chatDatas
    }
    //Save files
    func saveGroupFileshare(_ filesInfo : EnxSharedFileModel){
        guard var groupChat = chatDatas["groupchat"] else {
            chatDatas["groupchat"] = [filesInfo]
            return
        }
        groupChat.append(filesInfo)
        chatDatas["groupchat"] = groupChat
    }
    func updateGroupFileShare(_ filesInfo : EnxSharedFileModel){
        for model in chatDatas["groupchat"]!{
            if(model.isKind(of: EnxSharedFileModel.self)){
                let filterValue = model as! EnxSharedFileModel
                if(filterValue.text == filesInfo.text){
                    var groupFiles = chatDatas["groupchat"]
                    filesInfo.fileType = filterValue.fileType
                    let index = groupFiles!.firstIndex{$0 === filterValue}
                    groupFiles!.remove(at: index!)
                    groupFiles!.insert(filesInfo, at: index!)
                    chatDatas["groupchat"] = groupFiles
                    break
                }
            }
        }
    }
    func savePrivateFileShare(_ filesInfo : EnxSharedFileModel){
        guard var priChat = chatDatas[filesInfo.receipientsId] else {
            chatDatas[filesInfo.receipientsId] = [filesInfo]
            return
        }
        priChat.append(filesInfo)
        chatDatas[filesInfo.receipientsId] = priChat
    }
    func updatePrivateFileShare(_ filesInfo : EnxSharedFileModel){
        for model in chatDatas[filesInfo.receipientsId]!{
            if(model.isKind(of: EnxSharedFileModel.self)){
                let filterValue = model as! EnxSharedFileModel
                if(filterValue.text == filesInfo.text){
                    var privateFiles = chatDatas[filesInfo.receipientsId]
                    filesInfo.fileType = filterValue.fileType
                    let index = privateFiles!.firstIndex{$0 === filterValue}
                    privateFiles!.remove(at: index!)
                    privateFiles!.insert(filesInfo, at: index!)
                    chatDatas[filesInfo.receipientsId] = privateFiles
                    break
                }
            }
        }
    }
    //Clear all chat history
    func cleanChatHistory(){
        if(!chatDatas.isEmpty){
            chatDatas.removeAll()
        }
    }
    //Get resource path
    func getPath() -> Bundle? {
        for bundle in Bundle.allFrameworks{
            if(bundle.bundlePath.hasSuffix("Enx_UIKit_iOS.framework")){
            return bundle
            }
        }
        return nil
    }
    //update state once call connectd
    func connectionEstablishSuccess(_ isConnected : Bool){
        self.isConnected = isConnected
    }
    func getTopOptionColor() -> UIColor{
        return topViewColor
    }
    func getBottomOptionColor() -> UIColor{
        return bottomViewColor
    }
    //Create top options list
    func getTopOptionList() -> [UIButton]{
        if(topOptionList.isEmpty){
            topOptionList = createTopCustomeOptionButton(["user","request","menu"])
        }
        return topOptionList
    }
    //Create bottom options list
    func getBottomOptionList() -> [UIButton]{
        if(bottomOptionList.isEmpty){
            bottomOptionList = createBottomCustomeOptionButton(["audio","video","cameraswitch","audioswitch","chat","disconnect"])
        }
        
        return bottomOptionList
    }
    //Create menu option
    func getMenuOpt() -> [EnxMenuOptionObject]{
        if(moreOptionList.isEmpty){
            optList.count > 0 ? createMenuList(optList) : createMenuList([.recording,.muteRoom,.switchAT,.annotation])
        }
        return moreOptionList
    }
    //Create participant List option
    func getParticipantOptList () -> [ParticipantListOptions]{
        if(participentListOpt.isEmpty){
            participentListOpt = [.audio,.video,.chat,.disconnect]
        }
        return participentListOpt
    }
    //Update menu option after click on menu
    func updateMenuOptions( _ isSelected : Bool , eventType : MenuOptions){
        for (index, optionItem) in moreOptionList.enumerated(){
            if(optionItem.optionTag == eventType){
                moreOptionList.remove(at: index)
                optionItem.isSelected = isSelected
                if(eventType == .annotation){
                    if(isSelected){
                        optionItem.name = "Stop Annotation"
                    }
                    else{
                        optionItem.name = "Start Annotation"
                    }
                }
                moreOptionList.insert(optionItem, at: index)
                NotificationCenter.default.post(name: Notification.Name("updateMenu"), object: optionItem, userInfo: nil)
                break
            }
        }
    }
    func updateRoomMode(withMode : String){
        self.roomMode = withMode
    }
    func updateUserRole(withRole : Bool){
        isModerator = withRole
    }
    //Public method for app user to update bottom Option View Background Color
    @objc
    public func updateBottomOptionView(withColor :UIColor){
        self.bottomViewColor = withColor
        if(isConnected){
            NotificationCenter.default.post(name: Notification.Name(customeNotification.BottomViewColorUpdate.rawValue), object: withColor, userInfo: nil)
        }
    }
    //Public method for app user to update top Option View Background Color
    @objc
    public func updateTopOptionView(withColor :UIColor){
        self.topViewColor = withColor
        if(isConnected){
            NotificationCenter.default.post(name: Notification.Name(customeNotification.TopViewColorUpdate.rawValue), object: withColor, userInfo: nil)
        }
    }
    //Public method for app user to configure owne button for bottom options
    @objc
    public func configureBottomOptionList(withList : [UIButton]){
        self.bottomOptionList = withList
        if(isConnected){
            NotificationCenter.default.post(name: Notification.Name(customeNotification.BottomViewConfigurList.rawValue), object: withList, userInfo: nil)
        }
        //Need to handle max size
        
    }
    //Public method for app user to configure owne button for top options
    @objc
    public func configureTopOptionList(withList : [UIButton]){
        self.topOptionList = withList
        if(isConnected){
            NotificationCenter.default.post(name: Notification.Name(customeNotification.TopViewConfigurList.rawValue), object: withList, userInfo: nil)
        }
        //Need to handle max size
    }
    //Public method for app user to add custome button for there needs
    @objc
    public func addCustomButton(to list : [Int : UIButton]){
        self.customOpt = list
        if(isConnected){
            checkPossibilityForCustomeOpt()
        }
    }
    @objc
    public func configureParticipantList(to list : [ParticipantListOptions.RawValue]){
        if(participentListOpt.count > 0){
            participentListOpt.removeAll()
        }
        for item in list{
            participentListOpt.append(ParticipantListOptions(rawValue: item)!)
        }
    }
    //Check the possibility for user options
    private func checkPossibilityForCustomeOpt(){
        if(self.bottomOptionList.count >= 7 && self.topOptionList.count >= 7){
            NotificationCenter.default.post(name: Notification.Name(customeNotification.NoMoreOptionAllowed.rawValue), object: nil, userInfo: nil)
        }
        else if(self.bottomOptionList.count < 7 && self.topOptionList.count >= 7){
                let maxOption =  7 - self.bottomOptionList.count
            aditionaloption(position: .bottom, numberOf: maxOption > self.customOpt.count ? self.customOpt.count : maxOption)
            if(maxOption < self.customOpt.count){
                NotificationCenter.default.post(name: Notification.Name(customeNotification.FewOptionPending.rawValue), object: ["postion" : "bottom" , "item" : maxOption], userInfo: nil)
            }
        }
        else if(self.bottomOptionList.count >= 7 && self.topOptionList.count < 7){
                let maxOption =  7 - self.topOptionList.count
            aditionaloption(position: .top, numberOf: maxOption > self.customOpt.count ? self.customOpt.count : maxOption)
            if(maxOption < self.customOpt.count){
                NotificationCenter.default.post(name: Notification.Name(customeNotification.FewOptionPending.rawValue), object: ["postion" : "top" , "item" : maxOption], userInfo: nil)
            }
        }
        else{
            let maxButtomOption =  7 - self.bottomOptionList.count
            let maxTopOption =  7 - self.topOptionList.count
            if(self.customOpt.count >  (maxTopOption + maxButtomOption)){
                aditionaloption(position: .both, numberOf: (maxTopOption + maxButtomOption))
                    NotificationCenter.default.post(name: Notification.Name(customeNotification.FewOptionPending.rawValue), object: ["postion" : "total" , "item" : (maxTopOption + maxButtomOption)], userInfo: nil)
                }
            else{
                aditionaloption(position: .both, numberOf: self.customOpt.count)
            }
        }
    }
    private func aditionaloption(position : aditional , numberOf option : Int){
        if(position == .top){
            let keys = [Int](self.customOpt.keys)
            for i in 0..<option {
                if(self.topOptionList.count > keys[i]){
                    self.topOptionList.insert(self.customOpt[keys[i]]!, at: keys[i])
                }
                else{
                    self.topOptionList.insert(self.customOpt[keys[i]]!, at:self.topOptionList.count)
                }
            }
            NotificationCenter.default.post(name: Notification.Name(customeNotification.TopViewConfigurList.rawValue), object: self.topOptionList, userInfo: nil)
        }
        else if(position == .bottom){
            let keys = [Int](self.customOpt.keys)
                for i in 0..<option {
                    if(self.bottomOptionList.count > keys[i]){
                        self.bottomOptionList.insert(self.customOpt[keys[i]]!, at: keys[i])
                    }
                    else{
                        self.bottomOptionList.insert(self.customOpt[keys[i]]!, at:self.bottomOptionList.count)
                    }
                }
            NotificationCenter.default.post(name: Notification.Name(customeNotification.BottomViewConfigurList.rawValue), object: self.bottomOptionList, userInfo: nil)
        }
        else{
            let maxButtomOption =  7 - self.bottomOptionList.count
            let keys = [Int](self.customOpt.keys)
            for i in 0..<maxButtomOption {
                if(self.bottomOptionList.count > keys[i]){
                    self.bottomOptionList.insert(self.customOpt[keys[i]]!, at: keys[i])
                }
                else{
                    self.bottomOptionList.insert(self.customOpt[keys[i]]!, at:self.bottomOptionList.count)
                }
            }
            for i in maxButtomOption..<option {
                if(self.topOptionList.count > keys[i]){
                    self.topOptionList.insert(self.customOpt[keys[i]]!, at: keys[i])
                }
                else{
                    self.topOptionList.insert(self.customOpt[keys[i]]!, at:self.topOptionList.count)
                }
            }
            NotificationCenter.default.post(name: Notification.Name(customeNotification.BottomViewConfigurList.rawValue), object: self.bottomOptionList, userInfo: nil)
            NotificationCenter.default.post(name: Notification.Name(customeNotification.TopViewConfigurList.rawValue), object: self.topOptionList, userInfo: nil)
        }
        
    }
    //add additional button on user request 
    func isBothOptionHasCreted(){
        if(isTopOptCreated && isButtomOptCreated){
            if(self.customOpt.count > 0){
                checkPossibilityForCustomeOpt()
            }
            
        }
    }
    //default bottom option
    private func createBottomCustomeOptionButton(_ optionsName : [String]) ->[UIButton]{
        var buttonList : [UIButton] = []
        if let bundle = getPath(){
            for btnName in optionsName{
                let button = UIButton(type: .custom)
                //audio
                if(btnName.caseInsensitiveCompare("audio") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.audio.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "audio-on", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "audio-off", ofType: "png")!), for: .selected)
                }else if(btnName.caseInsensitiveCompare("video") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.video.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "video-on", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "video-off", ofType: "png")!), for: .selected)
                }else if(btnName.caseInsensitiveCompare("cameraswitch") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.cameraSwitch.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "camera-rotaion", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "camera-rotaion", ofType: "png")!), for: .selected)
                }else if(btnName.caseInsensitiveCompare("audioswitch") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.audioSwitch.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "earpiece", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "speaker", ofType: "png")!), for: .selected)
                }
                else if(btnName.caseInsensitiveCompare("chat") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.groupChat.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "groupChat", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "groupChat", ofType: "png")!), for: .selected)
                }
                else if(btnName.caseInsensitiveCompare("disconnect") == ComparisonResult.orderedSame){
                    button.tag = BottomOptions.disconnect.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "EndCall", ofType: "png")!), for: .normal)
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "EndCall", ofType: "png")!), for: .selected)
                }
                buttonList.append(button)
            }
        }
        isButtomOptCreated = true
        return buttonList
    }
    //default top options
    private func createTopCustomeOptionButton(_ optionsName : [String]) ->[UIButton]{
        var buttonList : [UIButton] = []
        if let bundle = getPath(){
            for btnName in optionsName{
                let button = UIButton(type: .custom)
                //Webinar Option
                if(btnName.caseInsensitiveCompare("request") == ComparisonResult.orderedSame){
                    button.tag = isModerator ? TopOptions.requestList.rawValue : TopOptions.requestFloor.rawValue
                    button.setImage( isModerator ? UIImage(contentsOfFile: bundle.path(forResource: "notification", ofType: "png")!) : UIImage(contentsOfFile: bundle.path(forResource: "raiseHand", ofType: "png")!), for: .normal)
                    if(roomMode != "group"){
                        buttonList.append(button)
                    }
                }//More option
                else if(btnName.caseInsensitiveCompare("menu") == ComparisonResult.orderedSame){
                    button.tag = TopOptions.menu.rawValue
                    button.setImage(UIImage(contentsOfFile: bundle.path(forResource: "more", ofType: "png")!), for: .normal)
                    buttonList.append(button)
                }
            }
        }
        isTopOptCreated = true
        return buttonList
    }
    //Create Menu List
    func createMenuList(_ menuList : [MenuOptions]) {
        for list in menuList{
            if(list == .recording){
                let recording = EnxMenuOptionObject()
                recording.name = "Start Recording"
                recording.isSelected = false
                recording.optionTag = .recording
                recording.isSwith = true
                if(isModerator){
                    moreOptionList.append(recording)
                }
            }
           else if(list == .muteRoom){
               let muteRoom = EnxMenuOptionObject()
               muteRoom.name = "Mute Room"
               muteRoom.isSelected = false
               muteRoom.optionTag = .muteRoom
               muteRoom.isSwith = true
               if(isModerator){
                   moreOptionList.append(muteRoom)
               }
            }
            else if(list == .switchAT){
                let switchAT = EnxMenuOptionObject()
                switchAT.name = "Switch Layout"
                switchAT.isSelected = false
                switchAT.optionTag = .switchAT
                switchAT.isSwith = true
                moreOptionList.append(switchAT)
             }
            else if(list == .annotation){
                let annotation = EnxMenuOptionObject()
                annotation.name = "Start Annotation"
                annotation.isSelected = false
                annotation.optionTag = .annotation
                annotation.isSwith = false
                moreOptionList.append(annotation)
             }
        }
    }
    //Public method for app user to configure owne screen share option
    @objc
    public func setOption(forScreenShare opt: [String : String]){
        guard opt == opt else { return }
        self.shareOption = opt
    }
    //Public method for app user to get configure  screen share option
    @objc
    public func getOptionForScreenShare() ->[String : String]?{
        return self.shareOption
    }
    func upDAteClientID(_ clientID : String){
        self.clientID = clientID
    }
    //Public method for app user to own client id
    @objc
    public func getClientID() -> String{
        return self.clientID
    }
    //Public method for app user to share app group name and key in case of screen share
    @objc
    public func setAppGroupsName( groupname : String , withUserKey keyName : String){
        EnxUtilityManager.shareInstance().setAppGroupsName(groupname, withUserKey: keyName)
    }
    func getDetailsFromService(_ serviceName : String){
        //let urlStr = serviceName + "/config"
        let url = URL(string: serviceName + "/config/index.json")
        let session = URLSession.shared
        let task = session.dataTask(with: url!) { [self] data, response, error in
            if error != nil {
                return
            }
            guard let httpResponse = response as? HTTPURLResponse,
                  (200...299).contains(httpResponse.statusCode) else {
                    print("Invalid Response received from the server")
                    return
                }
            guard let responseData = data else {
                    print("nil Data received from the server")
                    return
                    }
            do {
                    if let jsonResponse = try JSONSerialization.jsonObject(with: responseData, options: .mutableContainers) as? [String: Any] {
                        print("Main Parse data \(jsonResponse)")
                        if let hostData = jsonResponse["hosted_apps"] as? [String : Any]{
                            if let vcData = hostData["vc"] as? [String : Any]{
                                if let optionData = vcData["options"] as? [String : Any]{
                                    createButtomopt(optionData)
                                    createMenuopt(optionData)
                                    print("Parse data \(optionData)")
                                }
                            }
                        }
                        
                        } else {
                            print("data maybe corrupted or in wrong format")
                            throw URLError(.badServerResponse)
                        }
                    } catch let error {
                        print("JSON Parsing Error: \(error.localizedDescription)")
                    }
        }
        task.resume()
    }
    private func createButtomopt(_ optionlist : [String : Any]){
        var optList : [String] = []
        var isVideo : Bool = false
        var isAudio : Bool = false
        let iscameraswitch : Bool = true
        let isaudioswitch : Bool = true
        var ischat : Bool = false
        let isDisconnect : Bool = true
        for obj in optionlist.keys{
            if(obj.caseInsensitiveCompare("video_mute") == ComparisonResult.orderedSame){
                isVideo = (optionlist[obj] as! Bool)
            }else if(obj.caseInsensitiveCompare("audio_mute") == ComparisonResult.orderedSame){
                isAudio = (optionlist[obj] as! Bool)
            }else if(obj.caseInsensitiveCompare("group_chat") == ComparisonResult.orderedSame){
                ischat = (optionlist[obj] as! Bool)
            }
        }
        if(isVideo){
            optList.append("video")
        }
        if(isAudio){
            optList.append("audio")
        }
        if(iscameraswitch){
            optList.append("cameraswitch")
        }
        if(isaudioswitch){
            optList.append("audioswitch")
        }
        if(ischat){
            optList.append("chat")
        }
        if(isDisconnect){
            optList.append("disconnect")
        }
        bottomOptionList = createBottomCustomeOptionButton(optList)
    }
    private func createMenuopt(_ optionlist : [String : Any]){
        for obj in optionlist.keys{
            if(obj.caseInsensitiveCompare("mute_all") == ComparisonResult.orderedSame){
                if((optionlist[obj] as! Bool) == true){
                    optList.append(.muteRoom)
                }
            }else if(obj.caseInsensitiveCompare("recording") == ComparisonResult.orderedSame){
                if((optionlist[obj] as! Bool) == true){
                    optList.append(.recording)
                }
            }else if(obj.caseInsensitiveCompare("grid_mode_switch") == ComparisonResult.orderedSame){
                optList.append(.switchAT)
            }else if(obj.caseInsensitiveCompare("toolbar_setup") == ComparisonResult.orderedSame){
                if let subObj = optionlist[obj] as? [String : Any]{
                    if(subObj["annotation"] != nil){
                        optList.append(.annotation)
                    }
                }
            }
        }
        if(isConnected){
            createMenuList(optList)
        }
       
    }
}
